<?php 
/**
 * A parser class for based and advenced bbcodes.
 * @copyright (C) 2016 PanBB
 * @license http://www.gnu.org/licenses/gpl.html GPL version 2 or higher
 * @package PanBB
 * 
 * Reserved BBcode of PunBB
 * $tags = array('quote', 'code', 'b', 'i', 'u', 'color', 'colour', 'url', 'email', 'img', 'list', '*', 'h');
 * 
 * Supported Tags of Parser from PanBB
 * ['center', 'hide', 'right', 's', 'size', 'spoiler'];
 * 
 * Planned Tags
 * <abbr title="Common Gateway Interface, общий шлюзовый интерфейс">CGI</abbr>
 * <acronym title="Document Object Model, Объектная модель документа">DOM</acronym>
 * 
 */

class PanBBCode
{
	public $descr;
	public $bbcode = array('center','hide','right','s','size','spoiler','table','tbody','thead','tr','td','leftimg','rightimg');
	public $bb = array();
	public $code = false;
	public $hide = false;
	public $config = array(
		'hide' => true,
		'videoH' => '460',
		'videoW' => '608',
		'size' => array(8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30),
		'font' => array(
			'a_BraggaStars'=>'/mods/bb/font/a_BraggaStars.ttf',
			'a_FuturaRoundTitulSh'=>'/mods/bb/font/a_FuturaRoundTitulSh.ttf',
			'a_MachinaOrtoSht'=>'/mods/bb/font/a_MachinaOrtoSht.ttf',
			'Agatha_Modern'=>'/mods/bb/font/Agatha_Modern.ttf',
			'Art_Nouveau-Bistro'=>'/mods/bb/font/Art_Nouveau-Bistro.ttf',
			'Burlak'=>'/mods/bb/font/Burlak.ttf',
			'DSCrystal'=>'/mods/bb/font/DSCrystal.ttf',
			'HeinrichScript'=>'/mods/bb/font/HeinrichScript.ttf',
			'LehmannC'=>'/mods/bb/font/LehmannC.atf',
			'm_Ben_Krush'=>'/mods/bb/font/m_Ben_Krush.ttf',
			'WoodenShipDecorated'=>'/mods/bb/font/WoodenShipDecorated.ttf',
			'Arial'=>'',
			'Arial Black'=>'',
			'Century Gothic'=>'',
			'Courier New'=>'',
			'Georgia'=>'',
			'Impac'=>'',
			'System'=>'',
			'Tahoma'=>'',
			'Times New Roman'=>'',
			'Verdana'=>'',
			'Sans-Serif'=>''
		)
	);

	//Зачеркнутый текст
	public function s($val){
		return nl2br('<'.$val[1].'s>'.$val[5]);
	}
	//Расположение текста по центру
 	public function center($val){
		return nl2br(($val[1]?'</center>':'<center style="center">').$val[5]);
	}
	//Расположение текста по правому краю
	public function right($val){
		return nl2br(($val[1]?'</span>':'<span style="float:right;">').$val[5]);
	}
	//Скрытый текст
	public function hide($val){
		return nl2br(($val[1]?'</span></div>':'<div>Скрытый текст<span style="display:none;">').$val[5]);
	}
	//Раскрывающийся текст !!! Do not work (nternet Explorer/Firefox)
 	public function spoiler($val){
		return nl2br(($val[1]?'</details>':'<details>').$val[5]);
	}
	//Обтекание картинки текстом слева
	public function leftimg($val){
		return nl2br($val[1]?$val[5]:'<img alt="'.($val[4]?:$this->descr).'" title="'.($val[4]?:$this->descr).'" src="'.$val[5].'" class="leftimg" />');
	}
	//Обтекание картинки текстом справа
	public function rightimg($val){
		return nl2br($val[1]?$val[5]:'<img alt="'.($val[4]?:$this->descr).'" title="'.($val[4]?:$this->descr).'" src="'.$val[5].'" class="rightimg" />');
	}
	
	
	//Работающие расширенные теги
	//Размер шрифта от 8 до 30 пикселей
	public function size($val){
		return ($val[1]?'</span>':(in_array($val[4],$this->config['size'])?'<span style="font-size:'.$val[4].'px">':'<span>')).nl2br($val[5]);
	}
	
	
	//ТЕСТИРОВАНИЕ ТАБЛИЦЫ
	public function table($val){
		return nl2br('<'.$val[1].'table>'.$val[5]);
	}
	public function thead($val){
		return nl2br('<'.$val[1].'thead>'.$val[5]);
	}
	public function tbody($val){
		return nl2br('<'.$val[1].'tbody>'.$val[5]);
	}
	public function tr($val){
		return nl2br('<'.$val[1].'tr>'.$val[5]);
	}
	public function td($val){
		return nl2br('<'.$val[1].'td>'.$val[5]);
	}






	public function parse($text,$descr=''){
//		$text=htmlspecialchars(trim($text),ENT_QUOTES);//защита отключена, выводит весь HTML как текст
		$this->descr=$descr;//описание для тега картинок
		if(preg_match("(^([^\[\]]*)\[(/|)[a-z]+(|=(.*?))\])ui",$text,$d1)){//проверяем нужно ли парсить и заодно забираем весь текст до первого бб кода
			preg_match_all("(\[(/|)([a-z]+)(|=(.*?))\]([^\[\]]*))ui",$text,$d2,PREG_SET_ORDER);//разбираем текст на части
			$text=nl2br($d1[1]);//вставляем текст который нашли в начале
			foreach($d2 as $val){
				if(!in_array($val[2],$this->bbcode)){//проверяем разрешон ли бб код
					$text.=nl2br($val[0]);
					continue;
				}
				if($this->hide){//нужно для бб кода hide
					if(!$val[1] or $val[2]!='hide'){
					continue;
					}
				}
				if($val[1]){//проверка на закрытый бб код
					if(!$this->bb or end($this->bb)!=$val[2]){//проверка был ли открыт бб код
						$text.=nl2br($val[0]);
						continue;
					}
					array_pop($this->bb);
				}else{
					$this->bb[]=$val[2];
				}
				$text.=$this->$val[2]($val);//метод
				
			}
//			foreach($this->bb as $val){//закрываем не закрытые	//выдает ошибку
//				$text.=$this->$val(['','/',$val,'','','']);
//			}
		}
		return $text;
	}
}
